"""A generic form plug-in"""

category = "Form"
help_url = "manual/forms/custom"
icon = "os-form_base"
controls = [
    {
        "label": "Edit the script to modify the form",
        "type": "text"
    }
]
priority = 90
