"""OpenSesame core extensions"""

from libopensesame.metadata import __version__

# The name of the packages to check for updates on conda and pip
packages = ['opensesame', 'opensesame-core', 'datamatrix', 'qdatamatrix',
            'python-pseudorandom', 'pseudorandom', 'python-qnotifications',
            'qnotifications', 'python-fileinspector', 'fileinspector',
            'pyqode.python', 'pyqode.core', 'pyqode3-python', 'pyqode3-core']
