"""Graphical variable inspector"""

icon = "os-inspector"
label = "Show variable inspector"
shortcut = "Ctrl+I"
checkable = True
menu = {
    "index": 8,
    "submenu": "View"
}
toolbar = {
    "index": 11
}
settings = {
    "variable_inspector_visible": False,
    "variable_inspector_focus_shortcut": "Ctrl+5"
}
tooltip = "Show variable inspector"
