"""Preloads items in the background for smoother use"""

icon = "applications-accessories"
label = "Preload items"
settings = {
    "preload_items_interval": 1000,
    "preload_items_idle_threshold": 0.01

}
