* {
	font-family: 'Roboto', sans-serif;;
	line-height:  1.428571429;
	color: #263238;
}

body {
	margin: 0px;
	padding: 12px;
}

table tr td {
	vertical-align: top;
	padding-right: 36px;
}

.highlight span, code, pre {
	font-family: 'Roboto Mono', monospace!important;
	word-wrap: break-word;
}

a {
	color: #0277bd;
	text-decoration: none;
}

a:hover {
	color: #ff5722;
}

h1, h2 {
	font-family: 'Roboto Slab', serif;
	font-weight: normal;
	color: #0277bd!important;
}

h1 {
	font-size: 2em;
	display: block;
	height: 48px;
	border-bottom: solid 1px #ff5722;
}

h2 {
	font-size: 1.5em;
	border-bottom: solid 1px #cfd8dc;
}

.important-button,
.dismiss-button,
.button {
	/*font-family: 'Roboto Condensed';*/
	display: inline-block;
	padding: 4px 8px 4px 8px;
	margin: 4px 0px 4px 0px;
	text-decoration: none;
	border-radius: 4px;
	border: solid 1px;
}

.button {
	color: #0277bd;
	border-color: #cfd8dc;
}

.button:hover {
	color: #0277bd!important;
	background-color: #eceff1;
}

.dismiss-button {
	color: #fff;
	border-color: #2e7d32;
	background-color: #43A047;
}

.dismiss-button:hover {
	color: #fff;
	background-color: #388E3C;
}

.important-button {
	color: #fff;
	border-color: #0277bd;
	background-color: #039BE5;
}

.important-button:hover {
	color: #fff;
	background-color: #0288D1;
}

.footer {
	color: #78909c;
	font-size: 0.8em;
	border-top: solid 1px #555753;
	float: right;
	font-family: 'Roboto Condensed';
}

.footer a {
	font-family: 'Roboto Condensed';
}
