"""Integrates the menu into the toolbar."""

checkable = True
icon = "applications-system"
label = "Integrate menu and toolbar"
menu = {
    "index": -1,
    "separator_after": False,
    "separator_before": True,
    "submenu": "View"
}
priority = -999999
settings = {
    "toolbar_menu_active": False
}
tooltip = "Integrates the menu into the toolbar"
